@@ -5,7 +5,7 @@ Tools for computing the matrix exponential and the integral.
 
 import warnings
 import numpy as np
-import scipy.sparse.linalg.matfuncs as mf
+import scipy.sparse.linalg as spla
 import scipy.linalg as la
 
 
@@ -15,6 +15,14 @@ try:
 except TypeError:
     pass
 
+mf = spla.matfuncs
+try:
+    # scipy version 1.7 or older?
+    mf._ExpmPadeHelper
+except AttributeError:
+    # scipy version 1.8 or newer
+    mf = spla._matfuncs
+
 
 class _ExpmIntPadeHelper(mf._ExpmPadeHelper):
     def __init__(self, A, structure=None, use_exact_onenorm=False):
